	// eigene Header-datei
#include "SPI.h"

	// eingebundene Header-Dateien im selben Pfad
#include "ports.h"
#include <avr/io.h>
#include "main.h"




	// allgemeine Dateien
#include "../../common_SW/ASCII/ASCII/ASCII.h"
#include "../../common_SW/display_eDIPTFT-43A/display_eDIPTFT-43A/display_eDIPTFT-43A.h"


	// Include-dateien vom Compiler
#include <string.h>				// fr Stringfunktionen









/************************************************************************
*                 CPU Clock						     7.3728 MHz		 	*
* SCL Frequency = ------------------------------- = ----------------	*
*                 Clock-Prescaler * SPI-Prescaler    1 * 128			*
*																		*
* SCL Frequency = 58 kHz (gemessener Takt: ca. 57,6 kHz => OK)			*
*																		*
* Das Display kann (ohne Pausen) nur bis 100 kHz betrieben werden!!		*
*																		*
*																		*
* Auszug aus Display-Datenblatt:										*
* Eine Datenbertragung ist bis zu 100 kHz mglich. Wenn jedoch			*
* zwischen den einzelnen Bytes whrend der bertragung Pausen von		*
* jeweils min. 100 s eingehalten werden, kann ein Byte	mit bis zu		*
* 3 MHz bertragen werden.												*
*																		*
* "Rudi Sigl" <rsigl@lcd-module.de>:									*
* der SS-Pin muss theoretisch gar nicht auf HI gehen, er dient nur zu	*
* Synchronisation. Wichtig ist der CLK-Pin der nach einem empfangenen 	*
* Byte 10us ruhig sein sollte, damit das eDIP das Anwortbyte 			*
* bereitstellen kann.													*
*																		*
* soll: > 10s															*
* ist:  18,58 s  => OK	(Messung bei f_SCL = 57,6 kHz)												*
*																		*
* Nachtrag 26.07.2008: Quarz ist nun 14,745 MHz. f_SCL ist daher nun	*
* 115 kHz !!!															*
*************************************************************************/


/****************************************************************
* Funktion:														*
* Initialisierung der SPI-Schnittstelle (fr Display 4x20).		*
* => OK	(22.03.2009)											*
****************************************************************/
void init_SPI_mastermode_display_DOG_3x16(void)
	{
	// MOSI und SCK und SS als Ausgnge definieren
	DDR_SPI_MOSI |= (1<<SPI_MOSI);						// Bit setzen
	DDR_SPI_SCK |= (1<<SPI_SCK);						// Bit setzen

	// MISO als Eingang definieren
	DDR_SPI_MISO &= ~(1<<SPI_MISO);		// Bit lschen


	// als Master definieren (Datenblatt Seite 165)
	// ACHTUNG: mu vor dem SPE-Bit definiert werden, sonst lt es sich nicht setzen
	SPCR |= (1<<MSTR);			// Bit setzen

	// Data order (Datenblatt Seite 165)
	// 1: LSB zuerst / 0: MSB zuerst
	SPCR &= ~(1<<DORD);		// Bit lschen (MSB zuerst)
//	SPCR |= (1<<DORD);		// Bit setzen (LSB zuerst)

	// Clock parity (Datenblatt Seite 165)
	// 1: SCK high = idle / 0: SCK low = idle
	//SPCR &= ~(1<<CPOL);			// Bit lschen
	SPCR |= (1<<CPOL);			// Bit setzen
	
	// Clock-Phase (Datenblatt Seite 165)
	//SPCR &= ~(1<<CPHA);			// Bit lschen
	SPCR |= (1<<CPHA);			// Bit setzen
	
	// SPI-Teilfaktor auf 128 setzen (definiert die Frequenz des SPI-Taktes)
	setze_SPI_Teilfaktor(128);
	
	// Interrupt sperren (Datenblatt Seite 164)
	SPCR &= ~(1<<SPIE);		// Bit lschen
	
	// SPI freigeben (Datenblatt Seite 164)
	SPCR |= (1<<SPE);			// Bit setzen
	
	return;
	}





/****************************************************************
* Funktion:														*
* Setzt den Teilfaktor der SPI-Schnittstelle.					*
* bergabewert: Teilfaktor										*
*																*
* => OK	(12.10.2008)											*
****************************************************************/

// Taktfrequenz (Datenblatt Seite 169  => Teiler 1:8)
void setze_SPI_Teilfaktor(unsigned char ucTeilfaktor)
	{

	if (ucTeilfaktor == 4)
		{
		SPSR &= ~(1<<SPI2X);		// Bit lschen
		SPCR &= ~(1<<SPR1);		// Bit lschen
		SPCR &= ~(1<<SPR0);		// Bit lschen
		}

	if (ucTeilfaktor == 16)
		{
		SPSR &= ~(1<<SPI2X);		// Bit lschen
		SPCR &= ~(1<<SPR1);		// Bit lschen
		SPCR |= (1<<SPR0);			// Bit setzen
		}

	if (ucTeilfaktor == 64)
		{
		SPSR &= ~(1<<SPI2X);		// Bit lschen
		SPCR |= (1<<SPR1);			// Bit setzen
		SPCR &= ~(1<<SPR0);		// Bit lschen
		}

	if (ucTeilfaktor == 128)
		{
		SPSR &= ~(1<<SPI2X);		// Bit lschen
		SPCR |= (1<<SPR1);			// Bit setzen
		SPCR |= (1<<SPR0);			// Bit setzen
		}

	if (ucTeilfaktor == 2)
		{
		SPSR |= (1<<SPI2X);		// Bit setzen
		SPCR &= ~(1<<SPR1);		// Bit lschen
		SPCR &= ~(1<<SPR0);		// Bit lschen
		}

	if (ucTeilfaktor == 8)
		{
		SPSR |= (1<<SPI2X);		// Bit setzen
		SPCR &= ~(1<<SPR1);		// Bit lschen
		SPCR |= (1<<SPR0);			// Bit setzen
		}

	if (ucTeilfaktor == 32)
		{
		SPSR |= (1<<SPI2X);		// Bit setzen
		SPCR |= (1<<SPR1);			// Bit setzen
		SPCR &= ~(1<<SPR0);		// Bit lschen
		}

	if (ucTeilfaktor == 64)
		{
		SPSR |= (1<<SPI2X);		// Bit setzen
		SPCR |= (1<<SPR1);			// Bit setzen
		SPCR |= (1<<SPR0);			// Bit setzen
		}

	return;
	}






/****************************************************************
* Funktion:														*
* Sendet ein Datenbyte auf der SPI-Schnittstelle.				*
* Slave Select wird whrend dem Senden auf Low geschaltet		*
*																*
* => OK	(          )											*
****************************************************************/
void SPI_master_transmit (unsigned char ucData, unsigned char ucSelector)
	{
	unsigned char uiZaehler;
	volatile unsigned char ucTemp;

	PORT_CS_DISPLAY_DOG_3x16 &= ~(1<<CS_DISPLAY_DOG_3x16);


	// Warteschleife (mind. 60ns warten (wegen Display 4x20)
	// gemessener Wert mit 100: ca. 112s
	for(uiZaehler= 0; uiZaehler <= 100 ; uiZaehler++)
		ucTemp++;

	// Datenbyte in Ausgangsregister laden (Senden starten)
	SPDR = ucData;

	// warten bis gesendet
	while (! (SPSR & (1<<SPIF)))
		;	// do nothing
		
	// Warteschleife
	for(uiZaehler= 0; uiZaehler <= 100 ; uiZaehler++)
		ucTemp++;


	// erst wieder auf High schalten, wenn der letzte Befehl gesendet wurde!!!
//	if ( (ucSelector == DISPLAY_4X20) && (SPI.ucDisplay4x20_letzter_befehl == 1) )
		PORT_CS_DISPLAY_DOG_3x16 |= (1<<CS_DISPLAY_DOG_3x16);

	return;
	}





/****************************************************************
* Funktion:														*
* Sendet ein Datenbyte auf der SPI-Schnittstelle.				*
* Slave Select mu anderweitig im Programm geschaltet werden!!!	*
* bergabewert:	zu sendendes Datenbyte							*
* Rckgabewert:	empfangenes Datenbyte							*
*																*
* => OK	(          )											*
****************************************************************/
unsigned char spi_fuer_display_DOG_3x16 (unsigned char ucData)
	{

	// Datenbyte in Ausgangsregister laden (Senden starten)
	SPDR = ucData;

	// warten bis gesendet
	while (! (SPSR & (1<<SPIF)))
		;	// do nothing
		
	return (SPDR);
	}


